"""Consts used by Speedtest.net."""

DOMAIN = "speedtestdotnet"
DATA_UPDATED = f"{DOMAIN}_data_updated"

SENSOR_TYPES = {
    "ping": ["Ping", "ms"],
    "download": ["Download", "Mbit/s"],
    "upload": ["Upload", "Mbit/s"],
}
